/**
 * asm_alu
 * Demo de operacoes logico aritmeticas em GNU Assembly (GAS)
 * 1-soma; 2-sub; 3-mult; 
 * 4-AND; 5-OR; 6-XOR; 
 * 7-deslocamento linear para esquerda; 8-deslocamento linear para direita
 * 9-deslocamento circular para direita;
 * 10-deslocamento aritmtico para direita;
 * 20-fim
 * Use o modo Debug para atribuir um dos valores no R4
 *  
 * Cdigo utilizado na disciplina EA871 - Segundo semestre de 2020
 * Autora: Wu Shin Ting
 * Data:  20/09/2020
 *
 * Nota 1: Doxygen nao suporta documentacao de codigos em assembly.
 * Nota 3: Sinopse de cada instrucao: ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/ARMv6-M.pdf (A6) 
 */

/* This assembly file uses GNU syntax */
	.text							@segmento de instrucoes  
	.section	.rodata				@segmento somente de leitura
	.align	2						@endereco alinhado a multiplos de 4 bytes
	
.LC0:
	.text						    @monta no segmento de instrucoes
	.align	1						@alinha os enderecos em multiplos de 2 bytes 
	.global	main					@<main> eh um simbolo global
	.type main function			    @<main> eh uma funcao
	
main:
	push {%r0,%r1,%r2,%r3,%r4,%lr}	@empilha
									@SP <- [lr]; SP <- [r4]; SP <- [r3]; SP <- [r2]; 
									@SP <- [r1]; SP <- [r0]
    ldr %r0,=varC					@carregue endereco da memoria
    								@r0 <- [varC]
 	ldr %r3,[%r0]					@carregue o conteudo do endereco
 									@r3 <- [[varC]] 									
    mov %r2,#6						@mova da fonte para destino
    								@R2 <- 6
    mov %r1,#2						@R1 <- 2

iteracao:
    cmp %r4,#1						@[R4]-<valor imediato> e atualiza CPSR
    								@[CPSR] atualizado com o resultado da subtracao
    blt iteracao					@desvie se <Less Than>
    								@se menor ou igual, desvie para <iteracao>
    beq soma						@se <EQual>, desvie para <iteracao>
									@se igual, desvie para <soma>
	cmp %r4,#2						@[CPSR] atualizado com o resultado da subtracao
	beq subt						@se igual, desvie para <subt>
	
	cmp %r4,#3						@[CPSR] atualizado com o resultado da subtracao
	beq mult						@se igual, desvie para <mult>

	cmp %r4,#4						@[CPSR] atualizado com o resultado da subtracao
	beq land						@se igual, desvie para <land>

	cmp %r4,#5						@[CPSR] atualizado com o resultado da subtracao
	beq lor							@se igual, desvie para <lor>

	cmp %r4,#6						@[CPSR] atualizado com o resultado da subtracao
	beq lxor						@se igual, desvie para <lxor>

	cmp %r4,#7						@[CPSR] atualizado com o resultado da subtracao
	beq llsl						@se igual, desvie para <llsl>

	cmp %r4,#8						@[CPSR] atualizado com o resultado da subtracao
	beq llsr						@se igual, desvie para <llsr>

	cmp %r4,#9						@[CPSR] atualizado com o resultado da subtracao
	beq lror						@se igual, desvie para <lror>

	cmp %r4,#10						@[CPSR] atualizado com o resultado da subtracao
	beq lasr						@se igual, desvie para <lasr>

	cmp %r4,#20						@[CPSR] atualizado com o resultado da subtracao
	beq fim							@se igual, desvie para <div>

	b iteracao						@desvie para <iteracao>
	
#Operacoes aritmeticas
soma: 
	mov %r3, %r2						@R3 <- [R2]
	add %r3,%r1						@soma dos conteudos de 2 registradores
									@R3 <- [R3]+ [R1]
	str %r3,[%r0]						@armazene o valor na memoria
									@varC <- [R3]
	b iteracao						@desvie para <iteracao>
subt:
	mov %r3, %r2					@R3 <- [R2]
	sub %r3,%r1						@subtracao dos conteudos de 2 registradores
									@R3 <- [R3]+ [R1]
	str %r3,[%r0,#4]				@varC+4 <- [R3]
	b iteracao						@desvie para <iteracao>
mult:
	mov %r3, %r2					@ R3 <- [R2]
	mul %r3,%r1						@ multiplicacao dos conteudos de 2 registradores
									@ R3 <- [R3] * [R1]
	str %r3,[%r0,#8]				@ varC+8 <- [R3]
	b iteracao						@ desvie para <iteracao>

#Operacoes logicas
land:
	mov %r3, %r2					@ R3 <- [R2]
	and %r3,%r1						@ AND bit-a-bit dos conteudos de 2 registradores
									@ R3 <- [R3] & [R1]
	str %r3,[%r0,#12]				@ varC+12 <- [R3]
	b iteracao						@ desvie para <iteracao>
lor:
	mov %r3, %r2					@ R3 <- [R2]
	orr %r3,%r1						@ OR bit-a-bit dos conteudos de 2 registradores
									@ R3 <- [R3] | [R1]
	str %r3,[%r0,#16]				@ varC+16 <- [R3]
	b iteracao						@desvie para <iteracao>
lxor:
	mov %r3, %r2					@ R3 <- [R2]
	eor %r3,%r1						@ XOR bit-a-bit dos conteudos de 2 registradores
									@ R3 <- [R3] ^ [R1]
	str %r3,[%r0,#20]				@ varC+20 <- [R3]
	b iteracao						@desvie para <iteracao>

#Operacoes de deslocamento	
llsl:
	mov %r3, %r2					@ R3 <- [R2]
	lsl %r3,%r1						@ Deslocamento linear para esquerda 
									@ R3 <- [R3] << [R1]
	str %r3,[%r0,#24]				@ varC+24 <- [R3]
	b iteracao						@desvie para <iteracao>
llsr:
	mov %r3, %r2					@ R3 <- [R2]
	lsr %r3,%r1						@ Deslocamento linear para direita
									@ R3 <- [R3] >> [R1]
	str %r3,[%r0,#28]				@ varC+28 <- [R3]
	b iteracao						@desvie para <iteracao>
lror:
	mov %r3, %r2					@ R3 <- [R2]
	ror %r3,%r1						@ Deslocamento circular para direita 
									@ R3 <- [R3] >> [R1]
	str %r3,[%r0,#32]				@ varC+32 <- [R3]
	b iteracao						@desvie para <iteracao>
lasr:
	mov %r3, %r2					@ R3 <- [R2]
	asr %r3,%r1						@ Deslocamento circular para direita com preservacao de sinal 
									@ R3 <- [R3] >> [R1]
	str %r3,[%r0,#36]					@ varC+36 <- [R3]
	b iteracao						@desvie para <iteracao>
	
fim:
	pop {%r0,%r1,%r2,%r3,%r4,%pc}
	
	.data
	.align 2						@ alinha os endereco em multiplos de 4 bytes
varC: @ Endereo da variavel 
	.word 0x1ffff000

	.end
	
